#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <iomanip>
#include <vector>
#include <sstream>
#include <stdexcept>
#include <pthread.h>
#include <signal.h>
#include <errlog.h>

#include "dome.h"
#include "defnames.h"

pthread_mutex_t Dome::Lock::mtx = PTHREAD_MUTEX_INITIALIZER;
Dome_CMD Dome::cmd = { std::string(program_cfg), false, false, false };
Dome* Dome::self_ptr = 0;
const int Dome::initLifeSec = 64;
const int Dome::parkLifeSec = 64;
pthread_t Dome::thread_id;

Dome::Dome(): Domecfg( cmd.cfgfile ), deviceDome( Domecfg, cmd.test_mode ),
    closeDome( Domecfg, deviceDome ), openDome( Domecfg, deviceDome ) {
    tableMode["open"]   = &openDome;
    tableMode["close"]  = &closeDome;
//    std::cerr << "Constructor Dome\n";
}

void Dome::create() {
    Lock lock;
    ErrlogSingl::instance().reopen();
    if(self_ptr == 0) self_ptr = new Dome();
}

void Dome::destroy() {
    Lock lock;       // To prevent commands execution
    if(self_ptr == 0) return;
    delete self_ptr; // during Dome destruction
    self_ptr = 0;
}

Dome* Dome::device_ptr() {
    Lock lock;
    if( self_ptr == 0 ) throw ErrPark(ERR_INI_OBJECT, "Dome object");
    return self_ptr;
}

int Dome::specify( const std::vector<std::string>& opts ) {
    for( unsigned int j = 0; j < opts.size(); j++ ) {
        if( opts.at(j) != "dome" ) throw ErrSyn( ERR_PROT_WRKWD, opts.at(j) );
    }
    return the->life();
}

int  Dome::stop( const std::vector<std::string>& opts ) {
    if( !opts.empty() && opts.at(0) != "now" ) throw ErrSyn( ERR_PROT_WRKWD, opts.at(0) );
    pthread_kill( thread_id, SIGUSR1 );       //   NOW
//    if( !opts.empty() )
    {                                         // stop == stop now 
        the->stop_request();
        return 1;
    }
    return the->life();
}

typedef void(Dome::*SETF_T)(const std::string&);
typedef std::pair<const std::string, SETF_T> SET_PAIR;

void Dome::set(const std::string& key, const std::string& val) {
    static SET_PAIR set_kwd[] =  {
        SET_PAIR ("dome"  , &Dome::setAction),
        SET_PAIR ("limit" , &Dome::setLimit),
        SET_PAIR ("power" , &Dome::setPower),
        SET_PAIR ("light" , &Dome::setLight)
    };
    static std::map<const std::string, SETF_T, Cmpi> setf(set_kwd, set_kwd + sizeof(set_kwd)/sizeof(set_kwd[0]));
    if( setf.find(key) == setf.end()) throw ErrSyn( ERR_PROT_WRKWD, val);
    (this->*setf[key])(val);
}

void Dome::setAction( const std::string &s ) {
    if( s.empty() ) throw ErrSyn( ERR_PROT_WRKWD, "no action" );
    the =  tableMode[s];
}

void Dome::setLimit( const std::string &s ) {
    int pos;
    std::istringstream str(s);
    str >> pos;
    (tableMode["open"])->set( pos );
}

void Dome::setPower( const std::string &s ) {
    if( s == "off" ) { deviceDome.power_off(); return; }
    if( s == "on"  ) { deviceDome.power_on(); return; }
    throw ErrSyn( ERR_PROT_WRKWD, s );
}

void Dome::setLight( const std::string &s ) {
    if( s == "off" ) { deviceDome.light_off(); return; }
    if( s == "on"  ) { deviceDome.light_on(); return; }
    throw ErrSyn( ERR_PROT_WRKWD, s );
}

typedef std::string(Dome::*GETF_T)();
typedef std::pair<const std::string, GETF_T> GET_PAIR;

std::string Dome::get(const std::string& key) {
    static GET_PAIR get_kwd[] =  {
        GET_PAIR ("dome",     &Dome::status ),
        GET_PAIR ("position", &Dome::position ),
        GET_PAIR ("power",    &Dome::power_stat ),
        GET_PAIR ("light",    &Dome::light_stat )
    };
    static std::map<const std::string, GETF_T, Cmpi> getf(get_kwd, get_kwd + sizeof(get_kwd)/sizeof(get_kwd[0]));
    if( getf.find(key) == getf.end()) throw ErrSyn( ERR_PROT_WRKWD, key);
    return (this->*getf[key])();
}

std::string Dome::power_stat() {
    return "POWER="+deviceDome.power_stat();
}

std::string Dome::light_stat() {
    return "LIGHT="+deviceDome.light_stat();
}

std::string Dome::status() {
    return "DOME="+deviceDome.status();
}

std::string Dome::position() {
    point<int> c = deviceDome.position();
    std::ostringstream str;
    str << "L_POS=" << std::fixed << c.x() << " R_POS=" << c.y();
    return str.str();
}

std::string Dome::limit() {
    int c = openDome.pos_opened();
    std::ostringstream str;
    str << "OPENED_POS=" << c;
    return str.str();
}

