#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <cmath>
#include <signal.h>
#include <vector>
#include <iostream>
#include <iomanip>
#include <stdexcept>

#include <rs485.h>
#include <astrotime.h>

#include "dome_device.h"

AstroDateTime utc;

const std::string   sect_device("Devices");
const std::string   subs_device("Actuators");
const std::string   subs_power("Power");

Device::Device( Config& cf,  bool tst ) try : device( cf.inp<std::string>( "General", "Common", "Device") ), rs485( device ) {
                    //     RS485
    power_control = std::auto_ptr<Power>( new Power( cf, rs485, subs_power ) );
    dome_controller = std::auto_ptr<Curtain>( new Curtain( cf, rs485, subs_device ) );
    close();                            //    - 
                                        //    
//    std::cerr << "Constructor Device" << std::endl;
}
catch ( const std::exception& e ) {
    ErrlogSingl::instance().wthrow( ERR_STD_EXCP, e.what() );
}

Device::~Device(void) {
    try {
        close();      //    - 
    }
    catch( const ErrFat& e ) {
        ErrlogSingl::instance().write( e );
    }
//    std::cerr << "Destructor Device" << std::endl;
}

void Device::close() try {
    dome_controller->close();
}
catch ( const std::exception& e ) {
//    ErrlogSingl::instance().wthrow( ERR_DEV_FUNC, "Problems in closing" );
    ErrlogSingl::instance().write( std::string("Problems in closing: ") + e.what() );
}

void Device::open( int p ) try {
    dome_controller->open( p );
    sleep(1);
}
catch ( const std::exception& e ) {
//    ErrlogSingl::instance().wthrow( ERR_DEV_FUNC, "Problems in opening" );
    ErrlogSingl::instance().write( std::string("Problems in opening: ") + e.what() );
}

void Device::power_off() try {
    power_control->main_off();
}
catch ( const std::exception& e ) {
    ErrlogSingl::instance().wthrow( ERR_DEV_FUNC, "Problems in MASS/DIMM shutdown" );
}

void Device::power_on() try {
    power_control->main_on();
}
catch ( const std::exception& e ) {
    ErrlogSingl::instance().wthrow( ERR_DEV_FUNC, "Problems in MASS/DIMM start" );
}

void Device::light_off() try {
    power_control->netp_off();
}
catch ( const std::exception& e ) {
    ErrlogSingl::instance().wthrow( ERR_DEV_FUNC, "Problems in illumination off" );
}

void Device::light_on() try {
    power_control->netp_on();
}
catch ( const std::exception& e ) {
    ErrlogSingl::instance().wthrow( ERR_DEV_FUNC, "Problems in illumination on" );
}

int Device::needed_time( int pos ) {
    return (int)rint(std::abs( pos - dome_controller->avgPosition() )/dome_controller->rate());
}

